/*
*	1. Assume that id of node begins with 0(consistent to the index in the graph structure).
*	2. At the beginning of each list, there is a blank/demma node.	
*	3. The randomization issue of "srand( time( NULL))".
*/


#include "Viral_Marketing.h"


#ifndef WIN32
/*
* GetCurTime gets the current rusage info.
*
* @curTime stores the rusage info.
*
*/
void GetCurTime( struct rusage* curTime)
{
	int ret = getrusage( RUSAGE_SELF, curTime);
	if( ret != 0)
	{
		fprintf( stderr, "getrusage failure.\n");
		exit( 0);
	}
}

/*
* GetTime calculates the time information.
* 
* @timeStart, timeEnd indicate the rusages at the start and end, respectively.
* @userTime, sysTime get back the time information in the format of "float".
*
*/
void GetTime( struct rusage* timeStart, struct rusage* timeEnd, float* userTime, float* sysTime)
{
	(*userTime) = ((float)(timeEnd->ru_utime.tv_sec - timeStart->ru_utime.tv_sec)) + 
		((float)(timeEnd->ru_utime.tv_usec - timeStart->ru_utime.tv_usec)) * 1e-6;
	(*sysTime) = ((float)(timeEnd->ru_stime.tv_sec - timeStart->ru_stime.tv_sec)) +
		((float)(timeEnd->ru_stime.tv_usec - timeStart->ru_stime.tv_usec)) * 1e-6;
}

#endif



/*
*	Generate the probabilistic weight information for the IC-Model.
*/
float prob_weight_gen_IC( )
{
	float rtn;

	rtn = rand_f( 0, 1);
	if( rtn <= 0.25)
		rtn = 0.1;
	else if( rtn <= 0.5)
		rtn = 0.25;
	else if( rtn <= 0.75)
		rtn = 0.5;
	else
		rtn = 0.75;

	return rtn;
}


/*
*	prob_weight_adj_LT adjusts the probabilistic weight information for LT-Model.
*
* @g_prob_v indicates the probabilistic graph.
*/
void prob_weight_adj_LT( graph_t* g_prob_v)
{
	int i, node_n;
	int* in_degree;

	g_node_t* g_node_v;

	in_degree = (int*)malloc( g_prob_v->size * sizeof( int));
	memset( in_degree, 0, g_prob_v->size * sizeof( int));
/*s*/
	sta_v.memory_v += g_prob_v->size * sizeof( int);
	if( sta_v.memory_max < sta_v.memory_v)
		sta_v.memory_max = sta_v.memory_v;
/*s*/

//	g_node_v = (g_node_t*)malloc( sizeof( g_node_t));

	//Calculate the in-degree information.
	for( i=0; i<g_prob_v->size; i++)
	{
		g_node_v = g_prob_v->head_v[i].head->next;
		while( g_node_v != NULL)
		{
			node_n = g_node_v->id;
			in_degree[ node_n] ++;

			g_node_v = g_node_v->next;
		}
	}

	//Adjust the probabilistic weight information.
	for( i=0; i<g_prob_v->size; i++)
	{
		g_node_v = g_prob_v->head_v[i].head->next;
		while( g_node_v != NULL)
		{
			g_node_v->weight = float(1.0 / in_degree[ g_node_v->id]);

			g_node_v = g_node_v->next;
		}
	}

	//Release the malloced memory.
	free( in_degree);
/*s*/
	sta_v.memory_v -= g_prob_v->size * sizeof( float);
/*s*/
}

/*
*	Release the seed list.
*/
void release_seed_list( s_list_t* s_list_v)
{
	s_node_t* s_node_v1, *s_node_v2;
	s_node_v1 = s_list_v->s_head;

	while( s_node_v1->next != NULL)
	{
		s_node_v2 = s_node_v1;
		s_node_v1 = s_node_v1->next;
		free( s_node_v2);
/*s*/
		sta_v.memory_v -= sizeof( s_node_t);
/*s*/
	}
	
	free( s_node_v1);
	free( s_list_v);
/*s*/
	sta_v.memory_v -= sizeof( s_node_t) + sizeof( s_list_t);
/*s*/
}

/*
*	Construct a graph_t structure based on a graph file.
*
* @cfg indicates the configuratin information.
* @mode_n indicates the underlying model: 1 for IC-Model; 2 for LT-Model.
* 
* @return the corresponding graph_t structure with probabilities.
*/
graph_t* construct_graph( config_t* cfg, int mode_n)
{
	int i, node_n, edge_n, id1, id2;
	
	FILE* i_fp;

	graph_t*	g_prob_v;
	g_node_t*	g_node_v;

	if( (i_fp = fopen( cfg->g_file, "r")) == NULL)
	{
		fprintf( stderr, "Cannot open the graph data file.\n");
		exit( 0);
	}

	fscanf( i_fp, "%i%i", &node_n, &edge_n);

	//Initialize the graph structure.
	g_prob_v = (graph_t*)malloc( sizeof( graph_t));
	g_prob_v->head_v = (g_head_t*)malloc( cfg->node_n * sizeof( g_head_t));
	g_prob_v->size = node_n;
	g_prob_v->edge_n = edge_n;
	for( i=0; i<cfg->node_n; i++)
	{
		g_node_v = (g_node_t*)malloc( sizeof( g_node_t));
		memset( g_node_v, 0, sizeof( g_node_t));
		g_prob_v->head_v[i].head = g_node_v;
	}
/*s*/
	sta_v.memory_v += sizeof( graph_t) + 
						cfg->node_n * sizeof( g_head_t) + 
						cfg->node_n * sizeof( g_node_t);
/*s*/

	//Read & Construct the graph structure.
	for( i=0; i<cfg->edge_n; i++)
	{
		fscanf( i_fp, "%i%i", &id1, &id2);
		g_node_v = (g_node_t*)malloc( sizeof( g_node_t));
		g_node_v->id = id2;
		if( mode_n == 1)
			g_node_v->weight = prob_weight_gen_IC( );
		else
			g_node_v->weight = 0;		//The probabilistic weight would be adjusted later.

		g_node_v->next = g_prob_v->head_v[id1].head->next;
		g_prob_v->head_v[id1].head->next = g_node_v;
	}
/*s*/
	sta_v.memory_v += cfg->edge_n * sizeof( g_node_t);
	if( sta_v.memory_max < sta_v.memory_v)
		sta_v.memory_max = sta_v.memory_v;
/*s*/

	//Special-handling of probabilistic weights for LT-Model.
	if( mode_n == 2)
		prob_weight_adj_LT( g_prob_v);

	//Release related resources.
	fclose( i_fp);

	return g_prob_v;
}


/*
*	Release a graph_t structure.
*/
void release_graph( graph_t* graph_v)
{
	int i;

	g_node_t* g_node_v1, *g_node_v2;
	
	for( i=0; i<graph_v->size; i++)
	{
		g_node_v1 = graph_v->head_v[i].head;
		while( g_node_v1->next != NULL)
		{
			g_node_v2 = g_node_v1;
			g_node_v1 = g_node_v1->next;
			free( g_node_v2);
/*s*/
			sta_v.memory_v -= sizeof( g_node_t);
/*s*/
		}

		free( g_node_v1);
/*s*/
		sta_v.memory_v -= sizeof( g_node_t);
/*s*/
	}

/*s*/
	sta_v.memory_v -= graph_v->size * sizeof( g_head_t) +
						sizeof( graph_t);						
/*s*/
	free( graph_v->head_v);
	free( graph_v);
}

/* seed_additin_greedy1 achieves the minimum seed set while influence spread is at least J greedily.
*		(NO NEED TO USE THIS FUNCTION, WHICH IS DORMINATED BY GREEDY_LAZY)
*
* @g_prob_v indicates the probabilistic graph for IC-Model.
* @J indicates the requirement of influence spread that should be incurred.
* @sample_n indicates the sampling times.
* @model_n indicates the diffusion model: 1 for IC-Model, 2 for LT-Model.
* 
* @return the approximate minimum seed set(size).
*/
int seed_addition_greedy1( graph_t* g_prob_v, int J, int sample_n, int model_n)
{
	int i, max_inf_gain_id, rtn; 
	float inf_cnt, inf_v, inf_gain, max_inf_gain;
	int* seed_tag;

	s_list_t* s_list_v;
	s_node_t* s_node_v;

	//Initialize the seed tags.
	seed_tag = (int*)malloc( g_prob_v->size * sizeof( int));
	memset( seed_tag, 0, g_prob_v->size * sizeof( int));

	//Initialize the seed set;
	s_list_v = (s_list_t*)malloc( sizeof( s_list_t));
	memset( s_list_v, 0, sizeof( s_list_t));
	s_node_v = (s_node_t*)malloc( sizeof( s_node_t));
	memset( s_node_v, 0, sizeof( s_node_t));
	
	s_list_v->s_head = s_node_v;

	//Conduct the greedy process by adding the node that incurs the maximum marginal gain.
	inf_cnt = 0;
	while( inf_cnt < J)
	{
		max_inf_gain = 0;

		//Pick out the node that generates the maximum marginal gain.
		for( i=0; i<g_prob_v->size; i++)
		{
			//node with id=i has been slected as a seed.
			if( seed_tag[i] != 0)
				continue;
	
			//Add node i into the seed *temporarily*.
			s_node_v->next = (s_node_t*)malloc( sizeof( s_node_t));
			s_node_v->next->id = i;
			s_node_v->next->next = NULL;
			s_list_v->size ++;

			//Calculate the resulted influence spread after the addition of a new seed.
			if( model_n == 1)
				inf_v = calc_influence_spread_prob_IC( g_prob_v, s_list_v, sample_n);
			else
				inf_v = calc_influence_spread_prob_LT( g_prob_v, s_list_v, sample_n);
			
			//Compare the marginal gain.
			inf_gain = inf_v - inf_cnt > 0 ? inf_v - inf_cnt : 1;
			if( inf_gain > max_inf_gain)
			{
				max_inf_gain = inf_gain;
				max_inf_gain_id = i;
			}

			//Remove the temporary seed.
			free( s_node_v->next);
			s_node_v->next = NULL;
			s_list_v->size --;
		}

/*t*/
		if( i == g_prob_v->size && max_inf_gain == 0)
			break;
/*t*/

		//Update the seed set.
		s_node_v->next = (s_node_t*)malloc( sizeof( s_node_t));
		s_node_v = s_node_v->next;
		s_node_v->id = max_inf_gain_id;
		s_node_v->next = NULL;
		s_list_v->size ++;

		seed_tag[max_inf_gain_id] = 1;
		inf_cnt += max_inf_gain;		
	}
	
	rtn = s_list_v->size;

	//Release the malloced resources.
	free( seed_tag);
	release_seed_list( s_list_v);

	return rtn;
}

/* seed_addition_greedy1_lazy is same to seed_addition_greedy1 except for the lazy evaluation.
*	
* @Same interface as seed_addition_greedy1
*/
int seed_addition_greedy1_lazy( graph_t* g_prob_v, int J, int sample_n, int model_n)
{
	int i, tmp_n, h_leng_max, rtn, portion_n = 1;
	float inf_v, inf_cnt;

	b_heap_t* b_h;			//binary heap.

	s_list_t* s_list_v;		//seed set.
	s_node_t* s_node_v;

	//Initialize the seed set;
	s_list_v = (s_list_t*)malloc( sizeof( s_list_t));
	memset( s_list_v, 0, sizeof( s_list_t));
	s_node_v = (s_node_t*)malloc( sizeof( s_node_t));
	memset( s_node_v, 0, sizeof( s_node_t));
	
	s_list_v->s_head = s_node_v;
/*s*/
	sta_v.memory_v += sizeof( s_list_t) + sizeof( s_node_t);
/*s*/
	//Initialize the max-heap.
	b_h = (b_heap_t*)malloc( sizeof( b_heap_t));
	h_leng_max = g_prob_v->size + 1;
	b_h->h_arr = (int*)malloc( sizeof( int) * (h_leng_max + 1 ));
	b_h->obj_arr = (h_obj_t*)malloc( sizeof( h_obj_t) * (h_leng_max+1));
	b_h->size = h_leng_max + 1;
	b_h->rear = 0;
/*s*/
	sta_v.memory_v += sizeof( b_heap_t) + 
						(h_leng_max + 1) * sizeof( int) +
						(h_leng_max + 1) * sizeof( h_obj_t);
/*s*/
	//Enqueue all the ndoes.
	for( i=0; i<g_prob_v->size; i++)
	{
		//Construct the corresponding seed set *temporarily*.
		s_node_v->next = (s_node_t*)malloc( sizeof( s_node_t));
		s_node_v->next->id = i;
		s_node_v->next->next = NULL;
		s_list_v->size ++;

		//Calculate the respective gain.
		if( model_n == 1)
			inf_v = calc_influence_spread_prob_IC( g_prob_v, s_list_v, sample_n);
		else
			inf_v = calc_influence_spread_prob_LT( g_prob_v, s_list_v, sample_n);

		//Enqueue the max-queue with the key of gain.
		b_h->obj_arr[i+1].id = i;
		b_h->obj_arr[i+1].gain = inf_v;
		b_h->obj_arr[i+1].valid_tag = 1;
		insert_b_heap( b_h, i+1);

		//Remove the temporary seed.
		free( s_node_v->next);
		s_node_v->next = NULL;
		s_list_v->size --;
	}
/*t*
	print_b_heap( stdout, b_h);
/*t*/

	//Perform the greedy process.
	inf_cnt = 0;	
	while( inf_cnt < J && b_h->rear > 0)
	{
		//Update the invalid gain information.
		while( b_h->obj_arr[ tmp_n = get_top( b_h)].valid_tag == 0)
		{
			//Construct the seed set *temporarily*.
			s_node_v->next = (s_node_t*)malloc( sizeof( s_node_t));
			s_node_v->next->id = tmp_n - 1;
			s_node_v->next->next = NULL;
			s_list_v->size ++;

			//Caculate the corresponding gain.
			if( model_n == 1)
				inf_v = calc_influence_spread_prob_IC( g_prob_v, s_list_v, sample_n);
			else
				inf_v = calc_influence_spread_prob_LT( g_prob_v, s_list_v, sample_n);

			b_h->obj_arr[tmp_n].gain = inf_v - inf_cnt > 0 ? inf_v - inf_cnt : 1;
			b_h->obj_arr[tmp_n].valid_tag = 1;

			//Enqueue the current the node.
			insert_b_heap( b_h, tmp_n);

			//Remove the temporary seed.
			free( s_node_v->next);
			s_node_v->next = NULL;
			s_list_v->size --;
		}

		//Case: b_h->obj_arr[tmp_n].valid_tag == 1.
		//Update the seed set first.
		s_node_v->next = (s_node_t*)malloc( sizeof( s_node_t));
		s_node_v = s_node_v->next;
		s_node_v->id = tmp_n - 1;
		s_node_v->next = NULL;
		s_list_v->size ++;

		inf_cnt += b_h->obj_arr[tmp_n].gain;

		while( inf_cnt >= ceil((float)portion_n * J / MAX_STAGE_NUM))
		{
			printf( "Influence coverage #Stage %i.\n", portion_n);
			portion_n ++;
			
			if( portion_n > MAX_STAGE_NUM)
				break;
		}


/*s*/
		sta_v.memory_v += sizeof( s_node_t);
/*s*/
		//Update the valid_tags of nodes in the queue.
		for( i=1; i<=b_h->rear; i++)
		{
			b_h->obj_arr[ b_h->h_arr[i]].valid_tag = 0;
		}

	}//while(J)

	rtn = s_list_v->size;

	//Release the malloced structures.
	free( b_h->h_arr);
	free( b_h->obj_arr);
	free( b_h);
	release_seed_list( s_list_v);
/*s*/
	if( sta_v.memory_max < sta_v.memory_v)
		sta_v.memory_max = sta_v.memory_v;

	sta_v.memory_v -= ( g_prob_v->size + 2)* sizeof( int) +
						( g_prob_v->size + 2) * sizeof( h_obj_t) +
						sizeof( b_heap_t);
/*s*/
	return rtn;
}

/* seed_addition_greedy2 achieves the approximate size of the minimum seed set that can incur at least J influence spread.
*
* @Parameters are the same as those in seed_addition_greedy1.
*
* @return the approximate size of the minimum seed set.
*/
int seed_addition_greedy2( graph_t* g_prob_v, int J, int sample_n, int model_n)
{
	/*
	int i;
	float seed_cnt = 0;

	graph_t* g_deter_v;
	
	//Conduct samplings.
	for( i=0; i<sample_n; i++)
	{
		//Sample the probabilistic graph.
		g_deter_v = graph_sampling_IC( g_prob_v);
		
		//Solve the problem of "seed set minimization".on the deterministic graph.
		//seed_cnt += 
	}
	*/
	return 0;
}

/* seed_addition_greedy2_lazy achieves the approximate size of the minimum seed set can cover at least J influence spread 
*  with *lazy-evaluation* and *incremental* implementation.
*
*  @Same interface as that of seed_addition_greedy2.
*/
int seed_addition_greedy2_lazy( graph_t* g_prob_v, int J, int sample_n, int model_n)
{
	int i, portion_n = 1;
	float seed_cnt = 0;

	graph_t* g_deter_v;

	for( i=0; i<sample_n; i++)
	{
		while( i >= ceil( ((float)portion_n) * sample_n / MAX_STAGE_NUM - 1))
		{
			printf( "Sampling stage #%i: %i.\n", portion_n, i + 1);
			portion_n ++;
		}
		
		if( model_n == 1)				//IC-Model.
		{
			//Sample the probabilistic graph.
			g_deter_v = graph_sampling_IC( g_prob_v);

			//Conduct the "seed set minimization" process on the deterministic graph.
			seed_cnt += seed_addition_greedy_deter_IC( g_deter_v, J);

			//Release the deterministic graph.
			release_graph( g_deter_v);
		}
		else							//LT-Model.
		{
/*Added on 19-June-2011 by Cheng.*/

			//Sampling.
			graph_sampling_LT( g_prob_v);

			//Conduct the "set set minimization" process on the sampled graph for LT-Model.
			seed_cnt += seed_addition_greedy_deter_LT( g_prob_v, J);
		}
	}

	return (int)ceil( seed_cnt/sample_n);
}


/*------------------------Specific for IC-Model-------------------------*/

/* graph_sampling_IC samples the probabilistic graph into a deterministic on for IC-Model.
* 
* @g_prob_v indicates the probabilistic graph for IC-model.
*
* @Return one instance of deterministic graph from g_prob_v.
*/
graph_t* graph_sampling_IC( graph_t* g_prob_v)
{
	int i, edge_cnt = 0;
	float res;
	graph_t*	g_deter_v;
	g_node_t*	g_node_v1, *g_node_v2, *g_node_v3;

	//Allocate the memory for the deterministic graph.
	g_deter_v = (graph_t*)malloc( sizeof(graph_t));
	g_deter_v->size = g_prob_v->size;
	g_deter_v->head_v = (g_head_t*)malloc( sizeof( g_head_t) * g_prob_v->size);

	//Initialize the head node for each list.
	for( i=0; i<g_prob_v->size; i++)
	{
		g_deter_v->head_v[i].head = (g_node_t*)malloc( sizeof( g_node_t));
		memset( g_deter_v->head_v[i].head, 0, sizeof( g_node_t));
	}
/*s*/
	sta_v.memory_v += sizeof( graph_t) +
						g_prob_v->size * sizeof( g_head_t) +
						g_prob_v->size * sizeof( g_node_t);
/*s*/
	
	//Sample the probabilistic graph into a deterministic one.
	for( i=0; i<g_prob_v->size; i++)
	{
		g_node_v1 = g_prob_v->head_v[i].head->next;
		g_node_v2 = g_deter_v->head_v[i].head;

		while( g_node_v1 != NULL)
		{			
			res = rand_f( 0, 1);
			if( res <= g_node_v1->weight)
			{
				//Remain edge with probability equal to 1.
				g_node_v3 = (g_node_t*)malloc( sizeof( g_node_t));
				g_node_v3->id = g_node_v1->id;
				g_node_v3->next = NULL;
				//Ignore the weight information.
				
				g_node_v2->next = g_node_v3;
				
				g_node_v2 = g_node_v2->next;

				edge_cnt ++;
/*s*/
				sta_v.memory_v += sizeof( g_node_t);
/*s*/
			}
			
			g_node_v1 = g_node_v1->next;
		}//while
	}//for
/*s*/
	if( sta_v.memory_max < sta_v.memory_v)
		sta_v.memory_max = sta_v.memory_v;
/*s*/
	g_deter_v->edge_n = edge_cnt ;

	return g_deter_v;
}

/* calc_influence_spread_deter_IC calculates the influence spread on a deterministic graph given a seed set under IC-Model.
*
* @g_deter_v indicates the deterministic graph.
* @seed_v indicates the seed set.
*
* @return the incurred influence spread.
*/
float calc_influence_spread_deter_IC( graph_t* g_deter_v, s_list_t* seed_v)
{
	int i, sta = 0, rear = 0, cur_idx, chld_idx, inf_res;

	int* inf_tag;	//Tags for influence spread.
	int* n_que;		//The queue for BFS traversal.

	g_node_t* g_node_v;
	s_node_t* s_node_v;

	inf_tag = (int*)malloc( g_deter_v->size * sizeof( int));
	memset( inf_tag, 0, g_deter_v->size * sizeof( int));

	n_que = (int*)malloc( g_deter_v->size * sizeof( int));
	memset( n_que, 0, g_deter_v->size * sizeof( int));
/*s*/
	sta_v.memory_v += g_deter_v->size * sizeof( int) +
						g_deter_v->size * sizeof( int);
	if( sta_v.memory_max < sta_v.memory_v)
		sta_v.memory_max = sta_v.memory_v;
/*s*/
	
	//Enqueue all the seeds.
	inf_res = 0;
	s_node_v = seed_v->s_head->next;
	for( i=0; i<seed_v->size; i++)
	{
		n_que[ rear++] = s_node_v->id;
		inf_tag[ s_node_v->id] = 1;
		inf_res ++;

		s_node_v = s_node_v->next;
	}

	//Conduct the BFS traversal process.
	while( sta != rear)
	{
		cur_idx = n_que[sta];
		
		//Equeue the children of the current node.
		g_node_v = g_deter_v->head_v[cur_idx].head->next;
		while( g_node_v != NULL)
		{
			chld_idx = g_node_v->id;
			if( inf_tag[ chld_idx] == 0)
			{
				n_que[ rear++] = chld_idx;
				inf_tag[ chld_idx] = 1;
				inf_res ++;
			}

			g_node_v = g_node_v->next;
		}

		sta ++;
	}//while

	//Release malloced resources
	free( n_que);
	free( inf_tag);
/*s*/
	sta_v.memory_v -= g_deter_v->size * sizeof( int) +
						g_deter_v->size * sizeof( int);
/*s*/
	return float(inf_res);
}

/* calc_influence_spread_prob_IC calculates the influence spread on a probabilistic graph given a seed set under IC-Model.
*
* @g_prob_v indicates the probabilistic graph.
* @seed_v indicates the seed set.
* @sample_n indicates the sampling times.
*
* @return the incurred influence spread.
*/
float calc_influence_spread_prob_IC( graph_t* g_prob_v, s_list_t* seed_v, int sample_n)
{
	int i;
	float inf_cnt = 0;
	
	graph_t* g_deter_v;

	//Perform the sampling.
	for( i=0; i<sample_n; i++)
	{
		//Obtain the deterministic graph.
		g_deter_v = graph_sampling_IC( g_prob_v);

		//Calculate the influence spread on the deterministic graph.
		inf_cnt += calc_influence_spread_deter_IC( g_deter_v, seed_v);

		//Release the generated deterministic graph.
		release_graph( g_deter_v);
	}

	return inf_cnt/sample_n;
}

/* calc_influence_gain_on_deter_IC calculates the influence gain of adding a new node on a deterministic graph.
*
* @g_deter_v indicates the deterministic graph.
* @cur_inf indicates the current influence spread.
* @node_n indicates the new node to be added.
*
* @return the influence gain.
*/
float calc_influence_gain_on_deter_IC( graph_t* g_deter_v, int* cur_inf, int node_n)
{
	int cur_n, chld_n, sta = 0, rear = 0;
	int *n_que, *inf_arr;

	float gain_cnt = 0;

	g_node_t* g_node_v;
	
	//Initialize the influence arrays and queues.
	inf_arr = (int*)malloc( g_deter_v->size * sizeof( int));
	memset( inf_arr, 0, g_deter_v->size * sizeof( int));

	n_que = (int*)malloc( g_deter_v->size * sizeof( int));
	memset( n_que, 0, g_deter_v->size * sizeof( int));
/*s*/
	sta_v.memory_v += g_deter_v->size * sizeof( int) +
						g_deter_v->size * sizeof( int);
	if( sta_v.memory_max < sta_v.memory_v)
		sta_v.memory_max = sta_v.memory_v;
/*s*/
	//Do traversal.
	if( cur_inf[node_n] == 0)
	{
		n_que[rear++] = node_n;

		inf_arr[ node_n] = 1;
		gain_cnt ++;
	}
	while( sta != rear)
	{
		cur_n = n_que[ sta];

		g_node_v = g_deter_v->head_v[ cur_n].head->next;
		while( g_node_v != NULL)
		{
			chld_n = g_node_v->id;
			if( cur_inf[chld_n] == 0 && inf_arr[chld_n] == 0)
			{
				//Enqueue.
				n_que[rear++] = chld_n;
				
				inf_arr[chld_n] = 1;
				gain_cnt ++;
			}
			
			g_node_v = g_node_v->next;
		}//while

		sta ++;
	}

	//Release the malloced memory.
	free( n_que);
	free( inf_arr);
/*s*/
	sta_v.memory_v -= g_deter_v->size * sizeof( int) +
						g_deter_v->size * sizeof( int);
/*s*/
	return gain_cnt;
}

/* update_cur_influence_on_deter_IC updates the current influence array after the seed addition.
*
* @Same interface as that of calc_influence_on_deter.
*/
void update_cur_influence_on_deter( graph_t* g_deter_v, int* cur_inf, int node_n)
{
	int cur_n, chld_n, sta = 0, rear = 0;
	int *n_que;

	g_node_t* g_node_v;

	n_que = (int*)malloc( g_deter_v->size * sizeof( int));
	memset( n_que, 0, g_deter_v->size * sizeof( int));
/*s*/
	sta_v.memory_v += g_deter_v->size * sizeof( int);
	if( sta_v.memory_max < sta_v.memory_v)
		sta_v.memory_max = sta_v.memory_v;
/*s*/

	if( cur_inf[ node_n] == 0)
	{
		n_que[rear++] = node_n;
		
		cur_inf[ node_n] = 1;
	}
	while( sta != rear)
	{
		cur_n = n_que[ sta];

		g_node_v = g_deter_v->head_v[ cur_n].head->next;
		while( g_node_v != NULL)
		{
			chld_n = g_node_v->id;
			if( cur_inf[chld_n] == 0)
			{
				//Enqueue.
				n_que[rear++] = chld_n;

				cur_inf[chld_n] = 1;
			}
			
			g_node_v = g_node_v->next;
		}//while

		sta ++;
	}

	//Release malloced strctures.
	free( n_que);
/*s*/
	sta_v.memory_v -= g_deter_v->size * sizeof( int);	
/*s*/
}

/* seed_addition_greedy_deter_IC solves the problem of "seed set minimization" on a deterministic graph for IC-Model in a greedy way.
*	
* @g_deter_v indicates the deterministic graph.
* @J indicates the requirement of influence spread.
*
* @return the size of minimum seed set.
*/
float seed_addition_greedy_deter_IC( graph_t* g_deter_v, int J)
{
	int i, tmp_n, h_leng_max;
	float inf_gain, inf_cnt, seed_cnt = 0;

	int* cur_inf;

	b_heap_t* b_h;			//binary heap.

	//Initialize the influence structures.
	cur_inf = (int*)malloc( g_deter_v->size * sizeof( int));
	memset( cur_inf, 0, g_deter_v->size * sizeof( int));

	//Initialize the max-heap.
	b_h = (b_heap_t*)malloc( sizeof( b_heap_t));
	h_leng_max = g_deter_v->size + 1;
	b_h->h_arr = (int*)malloc( sizeof( int) * (h_leng_max + 1 ));
	b_h->obj_arr = (h_obj_t*)malloc( sizeof( h_obj_t) * (h_leng_max+1));
	b_h->size = h_leng_max + 1;
	b_h->rear = 0;
/*s*/
	sta_v.memory_v += g_deter_v->size * sizeof( int) +
						sizeof( b_heap_t) +
						(h_leng_max + 1) * sizeof( int) +
						(h_leng_max + 1) * sizeof( h_obj_t);
	if( sta_v.memory_max < sta_v.memory_v)
		sta_v.memory_max = sta_v.memory_v;
/*s*/
	//Enqueue all the ndoes.
	for( i=0; i<g_deter_v->size; i++)
	{
		//Calculate the respective gain.
		//inf_v = calc_influence_spread_prob_IC( g_deter_v, s_list_v, sample_n);
		inf_gain = calc_influence_gain_on_deter_IC( g_deter_v, cur_inf, i);

		//Enqueue the max-queue with the key of gain.
		b_h->obj_arr[i+1].id = i;
		b_h->obj_arr[i+1].gain = inf_gain;
		b_h->obj_arr[i+1].valid_tag = 1;
		insert_b_heap( b_h, i+1);
	}

	//Perform the greedy process.
	inf_cnt = 0;	
	while( inf_cnt < J && b_h->rear > 0)
	{
		//Update the invalid gain information.
		while( b_h->obj_arr[ tmp_n = get_top( b_h)].valid_tag == 0)
		{
			//Caculate the corresponding gain.
			//inf_v = calc_influence_spread_prob_IC( g_deter_v, s_list_v, sample_n);
			inf_gain = calc_influence_gain_on_deter_IC( g_deter_v, cur_inf, tmp_n-1);

			b_h->obj_arr[tmp_n].gain = inf_gain;
			b_h->obj_arr[tmp_n].valid_tag = 1;

			//Enqueue the current the node.
			insert_b_heap( b_h, tmp_n);
		}

		//Case: b_h->obj_arr[tmp_n].valid_tag == 1.
		seed_cnt ++;

		inf_cnt += b_h->obj_arr[tmp_n].gain;
		update_cur_influence_on_deter( g_deter_v, cur_inf, tmp_n-1);

		//Update the valid_tags of nodes in the queue.
		for( i=1; i<=b_h->rear; i++)
		{
			b_h->obj_arr[ b_h->h_arr[i]].valid_tag = 0;
		}

	}//while(J)
/*s*/
	if( sta_v.memory_max < sta_v.memory_v)
		sta_v.memory_max = sta_v.memory_v;
/*s*/
	//Release the malloced structures.
	free( b_h->h_arr);
	free( b_h->obj_arr);
	free( b_h);
	free( cur_inf);
/*s*/
	sta_v.memory_v -= (g_deter_v->size + 2) * sizeof( int) +
						(g_deter_v->size + 2) * sizeof( h_obj_t) +
						sizeof( b_heap_t) +
						g_deter_v->size * sizeof( int);
/*s*/

	return seed_cnt;
}


/*------------------------Specific for LT-Model-------------------------*/

/*
*	graph_sampling_LT is the counter-part of graph_sampling_IC for LT-Model.
*/
void graph_sampling_LT( graph_t* g_prob_v)
{
	int i;

	for( i=0; i<g_prob_v->size; i++)
	{
		g_prob_v->head_v[i].thr = rand_f( 0, 1);
	}
}

/*
*	calc_influence_spread_LT is the counter-part of calc_influence_spread_IC for LT-Model.
*/
float calc_influence_spread_prob_LT( graph_t* g_prob_v, s_list_t* seed_v, int sample_n)
{
	int i;
	float inf_cnt = 0;

	for( i=0; i<sample_n; i++)
	{
		//Sampling.
		graph_sampling_LT( g_prob_v);

		//Calculation.
		inf_cnt += calc_influence_spread_deter_LT( g_prob_v, seed_v);
	}

	return inf_cnt / sample_n;
}

/*
*	calc_influence_spread_deter_LT is the counter-part of calc_influence_spread_IC for LT-Model.
*/
float calc_influence_spread_deter_LT( graph_t* g_deter_v, s_list_t* seed_v)
{
	int sta = 0, rear = 0, inf_v = 0, cur_id, chld_id;
	int* inf_tag;
	int* n_que;
	float* inf_cnt;

	s_node_t* s_node_v;

	g_node_t* g_node_v;

	inf_tag = (int*)malloc( g_deter_v->size * sizeof( int));	
	memset( inf_tag, 0, g_deter_v->size * sizeof( int));
	n_que = (int*)malloc( g_deter_v->size * sizeof( int));
	memset( n_que, 0, g_deter_v->size * sizeof( int));
	inf_cnt = (float*)malloc( g_deter_v->size * sizeof( float));
	memset( inf_cnt, 0, g_deter_v->size * sizeof( float));

/*s*/
	sta_v.memory_v += g_deter_v->size * sizeof( int) +
						g_deter_v->size * sizeof( int) +
						g_deter_v->size * sizeof( float);
	if( sta_v.memory_max < sta_v.memory_v)
		sta_v.memory_max = sta_v.memory_v;
/*s*/

	//Enqueue/Activate all the seeds.
	s_node_v = seed_v->s_head->next;
	while( s_node_v != NULL)
	{
		n_que[ rear++] = s_node_v->id;
		inf_tag[ s_node_v->id] = 1;
		inf_v ++;

		s_node_v = s_node_v->next;
	}

	//Perform the spread propagation process.
	while( sta != rear)
	{
		cur_id = n_que[sta];
		g_node_v = g_deter_v->head_v[ cur_id].head->next;
		while( g_node_v != NULL)
		{
			chld_id = g_node_v->id;
			if( inf_tag[ chld_id] == 0)
			{
				inf_cnt[ chld_id] += g_node_v->weight;
				if( inf_cnt[ chld_id] >= g_deter_v->head_v[chld_id].thr)
				{
					n_que[ rear++] = chld_id;
					inf_tag[ chld_id] = 1;
					inf_v ++;
				}
			}

			g_node_v = g_node_v->next;
		}

		sta ++;
	}

	//Release malloced structures.
	free( inf_tag);
	free( n_que);
	free( inf_cnt);
/*s*/
	sta_v.memory_v -= g_deter_v->size * sizeof( int) +
						g_deter_v->size * sizeof( int) +
						g_deter_v->size * sizeof( float);
/*s*/

	return (float)inf_v;
}


/* seed_addition_greedy_deter_LT performs the seed_addition_greedy process on a deterministic graph for LT-Model.
*		(Counter-part to "seed_addition_greedge_deter_IC" for IC-Model.)
*
* @g_deter_v indicates the determimistic graph.
* @J indicates the requiremnt of influence spread.
*
* @return the average number of nodes.
*/
float seed_addition_greedy_deter_LT( graph_t* g_deter_v, int J)
{
	int i, max_inf_gain_id, rtn; 
	float inf_cnt, inf_v, inf_gain, max_inf_gain;
	int* seed_tag;

	s_list_t* s_list_v;
	s_node_t* s_node_v;

	//Initialize the seed tags.
	seed_tag = (int*)malloc( g_deter_v->size * sizeof( int));
	memset( seed_tag, 0, g_deter_v->size * sizeof( int));

	//Initialize the seed set;
	s_list_v = (s_list_t*)malloc( sizeof( s_list_t));
	memset( s_list_v, 0, sizeof( s_list_t));
	s_node_v = (s_node_t*)malloc( sizeof( s_node_t));
	memset( s_node_v, 0, sizeof( s_node_t));
	s_list_v->s_head = s_node_v;

/*s*/
	sta_v.memory_v += g_deter_v->size * sizeof( int) +
						sizeof( s_list_t) +
						sizeof( s_node_t);
	if( sta_v.memory_max < sta_v.memory_v)
		sta_v.memory_max = sta_v.memory_v;
/*s*/

	//Conduct the greedy process by adding the node that incurs the maximum marginal gain.
	inf_cnt = 0;
	while( inf_cnt < J)
	{
		max_inf_gain = 0;

		//Pick out the node that generates the maximal marginal gain.
		for( i=0; i<g_deter_v->size; i++)
		{
			//node with id=i has been slected as a seed.
			if( seed_tag[i] != 0)
				continue;
	
			//Add node i into the seed *temporarily*.
			s_node_v->next = (s_node_t*)malloc( sizeof( s_node_t));
			s_node_v->next->id = i;
			s_node_v->next->next = NULL;
			s_list_v->size ++;

			//Calculate the resulted influence spread after the addition of a new seed.
			inf_v = calc_influence_spread_deter_LT( g_deter_v, s_list_v);
			
			//Compare the marginal gain.
			inf_gain = inf_v - inf_cnt > 0 ? inf_v - inf_cnt : 1;
			if( inf_gain > max_inf_gain)
			{
				max_inf_gain = inf_gain;
				max_inf_gain_id = i;
			}

			//Remove the temporary seed.
			free( s_node_v->next);
			s_node_v->next = NULL;
			s_list_v->size --;
		}

		//No seeds are available. (Typical programming pattern after a loop.)
/*t*/
		if( i == g_deter_v->size && max_inf_gain == 0)	
		{
			break;
		}
/*t*/
		//Update the seed set.
		s_node_v->next = (s_node_t*)malloc( sizeof( s_node_t));
		s_node_v = s_node_v->next;
		s_node_v->id = max_inf_gain_id;
		s_node_v->next = NULL;
		s_list_v->size ++;

		seed_tag[max_inf_gain_id] = 1;
		inf_cnt += max_inf_gain;		
/*s*/	
		sta_v.memory_v += sizeof( s_node_t);
/*s*/
	}
	
	rtn = s_list_v->size;

/*s*/
	if( sta_v.memory_max < sta_v.memory_v)
		sta_v.memory_max = sta_v.memory_v;
	sta_v.memory_v -= g_deter_v->size * sizeof( int);
/*s*/

	//Release the malloced resources.
	free( seed_tag);
	release_seed_list( s_list_v);

	return (float)rtn;
}


/*------------------------------Baselines-------------------------------*/

/*
*	random_choose_seed chooses the seeds randomly until the incurred influence is at leat J.
*/
int random_choose_seed( graph_t* g_prob_v, int J, int sample_n, int model_n)
{
	int rand_v, seed_size, portion_n = 1; 
	float inf_cnt;
	int* seed_tag;

	s_list_t* s_list_v;
	s_node_t* s_node_v;

	//Initialize the seed tags.
	seed_tag = (int*)malloc( g_prob_v->size * sizeof( int));
	memset( seed_tag, 0, g_prob_v->size * sizeof( int));

	//Initialize the seed set;
	s_list_v = (s_list_t*)malloc( sizeof( s_list_t));
	memset( s_list_v, 0, sizeof( s_list_t));
	s_node_v = (s_node_t*)malloc( sizeof( s_node_t));
	memset( s_node_v, 0, sizeof( s_node_t));
	
	s_list_v->s_head = s_node_v;
/*s*/
	sta_v.memory_v += g_prob_v->size * sizeof( int) +
						sizeof( s_list_t) +
						sizeof( s_node_t);
/*s*/
	//Construct the seed set.
	inf_cnt = 0;
	while( inf_cnt < J)
	{
		//Choose a random seed.
		while( seed_tag[ rand_v = rand_i( 0, g_prob_v->size-1)] == 1);
	
		//Add the new seed into the seed list.
		s_node_v = (s_node_t*)malloc( sizeof( s_node_t));
		s_node_v->id = rand_v;
		s_node_v->next = s_list_v->s_head->next;
		s_list_v->s_head->next = s_node_v;

		s_list_v->size ++;
		seed_tag[ rand_v] = 1;
/*s*/
		sta_v.memory_v += sizeof( s_node_t);
/*s*/
		//Update the influence spread.
		if( model_n == 1)
			inf_cnt = calc_influence_spread_prob_IC( g_prob_v, s_list_v, sample_n);
		else
			inf_cnt = calc_influence_spread_prob_LT( g_prob_v, s_list_v, sample_n);

		while( inf_cnt >= ceil((float)portion_n * J / MAX_STAGE_NUM))
		{
			printf( "Influence coverage #Stage %i.\n", portion_n);
			portion_n ++;

			if( portion_n > MAX_STAGE_NUM)
				break;
		}
	}

	seed_size = s_list_v->size;
	
	//Release the malloced memory.
	free( seed_tag);
	release_seed_list( s_list_v);
/*s*/
	sta_v.memory_v -= g_prob_v->size * sizeof( int);
/*s*/
	return seed_size;
}

/*
*	Calculate the out-degree of each node in the graph.
*
* @g_prob_v indicates the graph.
*
* @return the out-degrees in form an array.
*/
int* calc_out_degree( graph_t* g_prob_v)
{
	int i;
	int* o_degree;

	g_node_t* g_node_v;

	o_degree = (int*)malloc( g_prob_v->size * sizeof(int));
	memset( o_degree, 0, g_prob_v->size * sizeof( int));
/*s*/
	sta_v.memory_v += g_prob_v->size * sizeof( int);
	if( sta_v.memory_max < sta_v.memory_v)
		sta_v.memory_max = sta_v.memory_v;
/*s*/

	for( i=0; i<g_prob_v->size; i++)
	{
		g_node_v = g_prob_v->head_v[i].head->next;
		while( g_node_v != NULL)
		{
			o_degree[i] ++;

			g_node_v = g_node_v->next;
		}
	}
	
	return o_degree;
}


/*
*	degree_oriented_choose_seed chooses the seeds based on nodes' degrees.
*/
int degree_oriented_choose_seed( graph_t* g_prob_v, int J, int sample_n, int model_n)
{
	int i, h_leng_max, cur_idx, seed_size, portion_n = 1;
	int* o_degree;
	float inf_cnt;

	b_heap_t* b_h;

	s_list_t* s_list_v;
	s_node_t* s_node_v;

	//Initialize the seed set;
	s_list_v = (s_list_t*)malloc( sizeof( s_list_t));
	memset( s_list_v, 0, sizeof( s_list_t));
	s_node_v = (s_node_t*)malloc( sizeof( s_node_t));
	memset( s_node_v, 0, sizeof( s_node_t));
	
	s_list_v->s_head = s_node_v;
/*s*/
	sta_v.memory_v += sizeof( s_list_t) + sizeof( s_node_t);
/*s*/
	//Initialize the max-heap.
	b_h = (b_heap_t*)malloc( sizeof( b_heap_t));
	h_leng_max = g_prob_v->size + 1;
	b_h->h_arr = (int*)malloc( sizeof( int) * (h_leng_max + 1 ));
	b_h->obj_arr = (h_obj_t*)malloc( sizeof( h_obj_t) * (h_leng_max+1));
	b_h->size = h_leng_max + 1;
	b_h->rear = 0;

/*s*/
	sta_v.memory_v += sizeof( b_heap_t) +
						(h_leng_max + 1) * sizeof( int) +
						(h_leng_max + 1) * sizeof( h_obj_t);
/*s*/
	//Enqueue all the node.
	o_degree = calc_out_degree( g_prob_v);
	for( i=0; i<g_prob_v->size; i++)
	{
		b_h->obj_arr[i].id = i + 1;
		b_h->obj_arr[i].gain = (float)o_degree[i];

		insert_b_heap( b_h, i+1);
	}

	//Perform the seed addition process.
	inf_cnt = 0;
	while( inf_cnt < J && b_h->rear > 0)
	{
		cur_idx = get_top( b_h) - 1;

		//Add the new seed into the seed list.
		s_node_v = (s_node_t*)malloc( sizeof( s_node_t));
		s_node_v->id = cur_idx;
		s_node_v->next = s_list_v->s_head->next;
		s_list_v->s_head->next = s_node_v;
		s_list_v->size ++;
/*s*/
		sta_v.memory_v += sizeof( s_node_t);
/*s*/
		//Calculate the incurred influence based on the new seed set.
		if( model_n == 1)
			inf_cnt = calc_influence_spread_prob_IC( g_prob_v, s_list_v, sample_n);
		else
			inf_cnt = calc_influence_spread_prob_LT( g_prob_v, s_list_v, sample_n);

		while( inf_cnt >= ceil((float)portion_n * J / MAX_STAGE_NUM))
		{
			printf( "Influence coverage #Stage %i.\n", portion_n);
			portion_n ++;

			if( portion_n > MAX_STAGE_NUM)
				break;
		}
	}

	seed_size = s_list_v->size;
	
	//Release the malloced memory.
	free( o_degree);
	free( b_h->h_arr);
	free( b_h->obj_arr);
	free( b_h);
	release_seed_list( s_list_v);
/*s*/
	sta_v.memory_v -= g_prob_v->size * sizeof( int) +
						g_prob_v->size * sizeof( int) +
						g_prob_v->size * sizeof( h_obj_t) +
						sizeof( b_heap_t);
/*s*/
	return seed_size;
}


/*
*	Calculate the centrality distacne of a specific node in the graph.
*/
float calc_centrality_node( graph_t* g_prob_v, int node_n)
{
	int i, sta = 0, rear = 0, dist_cnt, cur_idx, chld_idx;
	int* dist, *n_que;
	float cent_cnt;

	g_node_t* g_node_v;

	dist = (int*)malloc( g_prob_v->size * sizeof( int));
	memset( dist, 0, g_prob_v->size * sizeof( int));

	n_que = (int*)malloc( g_prob_v->size * sizeof( int));
	memset( n_que, 0, g_prob_v->size * sizeof( int));
/*s*/
	sta_v.memory_v += g_prob_v->size * sizeof( int) +
						g_prob_v->size * sizeof( int);
	if( sta_v.memory_max < sta_v.memory_v)
		sta_v.memory_max = sta_v.memory_v;
/*s*/

	//Perform the process of BFS.
	n_que[ rear++] = node_n;
	dist[ node_n] = 1;			//Used as a tag for traversal.
	dist_cnt = 1;
	while( sta != rear)
	{
		cur_idx = n_que[ sta];
		g_node_v = g_prob_v->head_v[ cur_idx].head->next;

		while( g_node_v != NULL)
		{
			chld_idx = g_node_v->id;
			if( dist[ chld_idx] == 0)
			{
				n_que[ rear++] = chld_idx;
				dist[ chld_idx] = dist_cnt;
			}

			g_node_v = g_node_v->next;
		}
		
		sta ++;
		dist_cnt ++;
	}
	
	//Assign the centralities of the nodes un-reachable from node_n with the size of graph.
	for( i=0; i<g_prob_v->size; i++)
	{
		if( dist[ i] == 0)
			cent_cnt += g_prob_v->size;
		else
			cent_cnt += (float)dist[i];
	}
	cent_cnt -= dist[ node_n];

	//Release the malloced structures.
	free( dist);
	free( n_que);

/*s*/
	sta_v.memory_v -= g_prob_v->size * sizeof( int) +
						g_prob_v->size * sizeof( int);
/*s*/
	//Collect the centrality information.
	return cent_cnt/g_prob_v->size;	
}


/*
*	Calculate the centrality distance of each node.
*/
float* calc_centrality_graph( graph_t* g_prob_v)
{
	int i;
	float* cent;

	cent = (float*)malloc( g_prob_v->size * sizeof( float));
	memset( cent, 0, g_prob_v->size * sizeof( float));

/*s*/
	sta_v.memory_v += g_prob_v->size * sizeof( float);
	if( sta_v.memory_max < sta_v.memory_v)
		sta_v.memory_max = sta_v.memory_v;
/*s*/

	for( i=0; i<g_prob_v->size; i++)
	{
		cent[i] = calc_centrality_node( g_prob_v, i);
	}

	return cent;

}

/*
*	centrality_oriented_choose_seed chooses the seeds based on nodes' centrality distances.
*/
int centrality_oriented_choose_seed( graph_t* g_prob_v, int J, int sample_n, int model_n)
{
	int i, h_leng_max, cur_idx, seed_size, portion_n = 1;
	float inf_cnt;
	float* cent;

	b_heap_t* b_h;

	s_list_t* s_list_v;
	s_node_t* s_node_v;

	//Initialize the seed set;
	s_list_v = (s_list_t*)malloc( sizeof( s_list_t));
	memset( s_list_v, 0, sizeof( s_list_t));
	s_node_v = (s_node_t*)malloc( sizeof( s_node_t));
	memset( s_node_v, 0, sizeof( s_node_t));
	
	s_list_v->s_head = s_node_v;
/*s*/
	sta_v.memory_v += sizeof( s_list_t) + sizeof( s_node_t);
/*s*/

	//Initialize the max-heap.
	b_h = (b_heap_t*)malloc( sizeof( b_heap_t));
	h_leng_max = g_prob_v->size + 1;
	b_h->h_arr = (int*)malloc( sizeof( int) * (h_leng_max + 1 ));
	b_h->obj_arr = (h_obj_t*)malloc( sizeof( h_obj_t) * (h_leng_max + 1));
	b_h->size = h_leng_max + 1;
	b_h->rear = 0;
/*s*/
	sta_v.memory_v += sizeof( b_heap_t) +
						(h_leng_max + 1) * sizeof( int) +
						(h_leng_max + 1) * sizeof( h_obj_t);
/*s*/

	//Enqueue all the node.
	cent = calc_centrality_graph( g_prob_v);
	for( i=0; i<g_prob_v->size; i++)
	{
		b_h->obj_arr[i].id = i + 1;
		b_h->obj_arr[i].gain = cent[i];

		insert_b_heap( b_h, i+1);
	}

	//Perform the seed addition process.
	inf_cnt = 0;
	while( inf_cnt < J && b_h->rear > 0)
	{
		cur_idx = get_top( b_h) - 1;

		//Add the new seed into the seed list.
		s_node_v = (s_node_t*)malloc( sizeof( s_node_t));
		s_node_v->id = cur_idx;
		s_node_v->next = s_list_v->s_head->next;
		s_list_v->s_head->next = s_node_v;
		s_list_v->size ++;

		//Calculate the incurred influence based on the new seed set.
		if( model_n == 1)
			inf_cnt = calc_influence_spread_prob_IC( g_prob_v, s_list_v, sample_n);
		else
			inf_cnt = calc_influence_spread_prob_LT( g_prob_v, s_list_v, sample_n);

		while( inf_cnt >= ceil((float)portion_n * J / MAX_STAGE_NUM))
		{
			printf( "Influence coverage #Stage %i.\n", portion_n);
			portion_n ++;

			if( portion_n > MAX_STAGE_NUM)
				break;
		}
	}

	seed_size = s_list_v->size;
	
	//Release the malloced memory.
	free( cent);
	free( b_h->h_arr);
	free( b_h->obj_arr);
	free( b_h);
	release_seed_list( s_list_v);
/*s*/
	sta_v.memory_v -= g_prob_v->size * sizeof( float) +
						g_prob_v->size * sizeof( int) +
						g_prob_v->size * sizeof( h_obj_t) +
						sizeof( b_heap_t);
/*s*/

	return seed_size;
}

/*
*	page_rank_choose_seed chooses the seeds based on the page_rank result.
*/
int page_rank_choose_seed( graph_t* g_deter_v, int J, int sample_n, int model_n)
{
	return 0;
}

/*
*	opt_brute_force searchs the optimal solution in a brute-force way.
*/
int opt_brute_force( graph_t* g_prob_v, int J, int sample_n, int model_n)
{
	int i, j, seed_cnt, cnt;
	int *occupy, *sta, *combi;

	float inf_v = 0;

	s_list_t* s_list_v;
	s_node_t* s_node_v;

	occupy = (int*)malloc( g_prob_v->size * sizeof( int));
	sta = (int*)malloc( g_prob_v->size * sizeof( int));
	combi = (int*)malloc( g_prob_v->size * sizeof( int));

/*s*/
	sta_v.memory_v += 3 * g_prob_v->size * sizeof( int);
/*s*/

	seed_cnt = 1;
	while( seed_cnt <= g_prob_v->size)
	{
		memset( occupy, 0, g_prob_v->size * sizeof( int));
		memset( sta, 0, g_prob_v->size * sizeof( int));

		//For each seed list with length of "seed_cnt"(via combination).
		cnt = 0;
		while( cnt >= 0)
		{
			while( cnt < seed_cnt)
			{
				i = sta[cnt];
				while( occupy[i] != 0 && i < g_prob_v->size)
					i++;
				
				if( i == g_prob_v->size)
				{
					for( j=cnt; j<seed_cnt; j++)
						sta[j] = sta[ cnt-1];			//sta[j] = 0;
					cnt --;
					if( cnt >= 0)
						occupy[ combi[cnt]] = 0;
					
					break;
				}
				
				combi[cnt] = i;
				occupy[i] = 1;
				sta[cnt] = i + 1;
				cnt++;
			}
			
			//Rechieve a seed list with the length of "seed_cnt".
			if( cnt == seed_cnt)
			{
				//Construct the corresponding seed set.
				s_list_v = (s_list_t*)malloc( sizeof( s_list_t));
				memset( s_list_v, 0, sizeof( s_list_t));
				s_node_v = (s_node_t*)malloc( sizeof( s_node_t));
/*s*/
				sta_v.memory_v += sizeof( s_list_t) + sizeof( s_node_t);
/*s*/
				s_node_v->next = NULL;
				s_list_v->s_head = s_node_v;

				for( i=0; i<seed_cnt; i++)
				{
					s_node_v = (s_node_t*)malloc( sizeof( s_node_t));
/*s*/	
					sta_v.memory_v += sizeof( s_node_t);
/*s*/
					s_node_v->id = combi[i];
					s_node_v->next = s_list_v->s_head->next;
					s_list_v->s_head->next = s_node_v;
					s_list_v->size ++;
				}

				//Perform the "influence spread" calculation.
				if( model_n == 1)
					inf_v = calc_influence_spread_prob_IC( g_prob_v, s_list_v, sample_n);
				else
					inf_v = calc_influence_spread_prob_LT( g_prob_v, s_list_v, sample_n);
				if( inf_v >= J)
				{
/*s*/
					if( sta_v.memory_max < sta_v.memory_v)
						sta_v.memory_max = sta_v.memory_v;
/*s*/
					release_seed_list( s_list_v);

					goto E;
				}				
				
				cnt --;
				occupy[ combi[cnt]] = 0;
			}
		}//while( cnt)

		seed_cnt ++;
	}//while( seed_cnt)

E:
	//Release malloced structure.
	free( occupy);
	free( sta);
	free( combi);

/*s*/
	sta_v.memory_v -= 3 * g_prob_v->size * sizeof( int);
/*s*/

	return seed_cnt;
}

/*-------------SCC(strongly connected component) APIs-------------------*/

/*
*	The recursive node visit on a graph node.
*/
void DFS_visit( graph_t* graph_v, int node_n, int &time_cnt, int* finish_t_arr, int scc_cur, int* scc_arr)
{
	int nei_idx;

	g_node_t* g_node_v;

	graph_v->head_v[ node_n].tag = 1;

	g_node_v = graph_v->head_v[ node_n].head->next;
	while( g_node_v != NULL)
	{
		nei_idx = g_node_v->id;
		if( graph_v->head_v[ nei_idx].tag == 0)
		{
			DFS_visit( graph_v, nei_idx, time_cnt, finish_t_arr, scc_cur, scc_arr);
		}

		g_node_v = g_node_v->next;
	}

	graph_v->head_v[node_n].tag = 2;
	scc_arr[ node_n] = scc_cur;
	finish_t_arr[ time_cnt++] = node_n;
}

/*
*	Do DFS on a graph.
*/
int* DFS( graph_t* graph_v)
{
	int i, time_cnt = 0, scc_cnt = 0;
	int* finish_t_arr, *scc_arr;

	scc_arr = (int*)malloc( graph_v->size * sizeof( int));			//Interface use.
	finish_t_arr = (int*)malloc( graph_v->size * sizeof( int));
	memset( finish_t_arr, 0, graph_v->size * sizeof( int));
/*s*/
	sta_v.memory_v += 2 * graph_v->size * sizeof( int);
	if( sta_v.memory_v < sta_v.memory_max)
		sta_v.memory_max = sta_v.memory_v;
/*s*/
	//Coloring each node in the graph.
	for( i=0; i<graph_v->size; i++)
	{
		graph_v->head_v[i].tag = 0;
	}

	for( i=0; i<graph_v->size; i++)
	{
		if( graph_v->head_v[i].tag == 0)
			DFS_visit( graph_v, i, time_cnt, finish_t_arr, scc_cnt, scc_arr);
	}
	
	free( scc_arr);
/*s*/
	sta_v.memory_v -= graph_v->size * sizeof( int);
/*s*/

	return finish_t_arr;
}

/*
*	Do the DFS on a graph w.r.t. the finishing times of nodes of the DFS of its transposed graph.
*/
int DFS_with_order( graph_t* graph_v, int* finish_t_arr, int* scc_arr)
{
	int i, node_n, scc_cnt = 0, time_cnt = 0;
	int* finish_t_arr_tmp;

	finish_t_arr_tmp = (int*)malloc( graph_v->size * sizeof( int));	//useless.
/*s*/
	sta_v.memory_v += graph_v->size * sizeof( int);
	if( sta_v.memory_max < sta_v.memory_v)
		sta_v.memory_max = sta_v.memory_v;
/*s*/
	//Coloring each node in the graph.
	for( i=0; i<graph_v->size; i++)
		graph_v->head_v[i].tag = 0;

	for( i=graph_v->size-1; i>=0; i--)
	{
		node_n = finish_t_arr[i];
		if( graph_v->head_v[ node_n].tag == 0)
		{
			DFS_visit( graph_v, node_n, time_cnt, finish_t_arr_tmp, scc_cnt, scc_arr);

			scc_cnt ++;
		}
	}

	free( finish_t_arr_tmp);
/*s*/
	sta_v.memory_v -= graph_v->size * sizeof( int);
/*s*/
	return scc_cnt;
}

/*
*	Transpose a graph.
*/
graph_t* transpose_graph( graph_t* graph_v)
{
	int i, nei_n;

	graph_t* graph_v1;
	g_node_t* g_node_v, *g_node_v1;

	//Initialize a new graph.
	graph_v1 = (graph_t*)malloc( sizeof( graph_t));
	graph_v1->size = graph_v->size;
	graph_v1->head_v = (g_head_t*)malloc( graph_v1->size * sizeof( g_head_t));
	for( i=0; i<graph_v1->size; i++)
	{
		g_node_v = (g_node_t*)malloc( sizeof( g_node_t));
		g_node_v->next = NULL;
		graph_v1->head_v[i].head = g_node_v;
	}
/*s*/
	sta_v.memory_v += sizeof( graph_t) +
						graph_v1->size * sizeof( g_head_t) +
						graph_v1->size * sizeof( g_node_t);
/*s*/

	//Transpose the graph.
	for( i=0; i<graph_v->size; i++)
	{
		g_node_v = graph_v->head_v[i].head->next;
		while( g_node_v != NULL)
		{
			nei_n = g_node_v->id;

			g_node_v1 = (g_node_t*)malloc( sizeof( g_node_t));
/*s*/
			sta_v.memory_v += sizeof( g_node_t);
/*s*/
			g_node_v1->id = i;
			g_node_v1->next = graph_v1->head_v[ nei_n].head->next;
			graph_v1->head_v[ nei_n].head->next = g_node_v1;

			g_node_v = g_node_v->next;
		}
	}
/*s*/
	if( sta_v.memory_max < sta_v.memory_v)
		sta_v.memory_max = sta_v.memory_v;
/*s*/

	return graph_v1;
}

/*
*	Calculate the strongly connected components in a graph.
*/
scc_t* calc_scc( graph_t* graph_v)
{
	int* finish_t_arr;

	graph_t* graph_v_tran;

	scc_t* scc_v;

	scc_v = (scc_t*)malloc( sizeof( scc_t));
	memset( scc_v, 0, sizeof( scc_t));
	scc_v->size = graph_v->size;
	scc_v->scc_arr = (int*)malloc( scc_v->size * sizeof( int));

/*s*/
	sta_v.memory_v += sizeof( scc_t) + scc_v->size * sizeof( int);
	if( sta_v.memory_max < sta_v.memory_v)
		sta_v.memory_max = sta_v.memory_v;
/*s*/

	//DFS on the original graph.
	finish_t_arr = DFS( graph_v);

	//Transpose the original graph.
	graph_v_tran = transpose_graph( graph_v);

	//DFS on the transposed graph w.r.t. the finishing times.
	scc_v->num = DFS_with_order( graph_v_tran, finish_t_arr, scc_v->scc_arr);

	//Release the malloced memory.
	free( finish_t_arr);
	release_graph( graph_v_tran);
/*s*/
	sta_v.memory_v -= graph_v->size * sizeof( int);
/*s*/

	return scc_v;
}

/*
*	The compare function for the qsort.
*/
int edge_comp( const void* edge1, const void* edge2)
{
	int rtn = 0;
	if( ((edge_t*)edge1)->idx1 > ((edge_t*)edge2)->idx1)
		rtn = 1;
	else if( ((edge_t*)edge1)->idx1 == ((edge_t*)edge2)->idx1 &&
				((edge_t*)edge1)->idx2 > ((edge_t*)edge2)->idx2 )
				rtn = 1;
	else if( ((edge_t*)edge1)->idx1 == ((edge_t*)edge2)->idx1 &&
				((edge_t*)edge1)->idx2 == ((edge_t*)edge2)->idx2 )
				rtn = 0;
	else
		rtn = -1;
	return rtn;
}

/*
*	Generate the edge set based on the SCC information.
*
* @graph_v indicates the original graph.
* @scc_v indicates the SCCs within the original graph.
*
* @return the edge set.(un-condensed)
*/
edge_s_t* gen_edge_set( graph_t* graph_v, scc_t* scc_v)
{
	int i, chld_idx, idx1, idx2;

	g_node_t* g_node_v;

	edge_s_t* edge_s_v;

	edge_s_v = (edge_s_t*)malloc( sizeof( edge_s_t));
	memset( edge_s_v, 0, sizeof( edge_s_t));
	edge_s_v->size = graph_v->edge_n;
	edge_s_v->edge_arr = (edge_t*)malloc( edge_s_v->size * sizeof( edge_t));
/*s*/
	sta_v.memory_v += sizeof( edge_s_t) + edge_s_v->size * sizeof( edge_t);
	if( sta_v.memory_v < sta_v.memory_v)
		sta_v.memory_v = sta_v.memory_max;
/*s*/

	//Construct the un-condensed edge set.
	for( i=0; i<graph_v->size; i++)
	{
		idx1 = scc_v->scc_arr[i];

		g_node_v = graph_v->head_v[i].head->next;
		while( g_node_v != NULL)
		{
			chld_idx = g_node_v->id;
			idx2 = scc_v->scc_arr[ chld_idx];

			edge_s_v->edge_arr[ edge_s_v->num].idx1 = idx1;
			edge_s_v->edge_arr[ edge_s_v->num++].idx2 = idx2;

			g_node_v = g_node_v->next;
		}
	}
/*t*/
	if( edge_s_v->num != graph_v->edge_n)
		fprintf( stderr, "The size of un-condensed edge set is inconsistent.\n");
/*t*/

	//Sorting the un-condensed edge set.
	qsort( edge_s_v->edge_arr, edge_s_v->num, sizeof( edge_t), edge_comp);

	//Condense the edge set.(Done in gen_graph_scc)

	return edge_s_v;

}

/*
*	Generate the new graph from the original graph based on the SCC information.
*
* @graph_v indicates the original graph.
* 
* @return the new generated graph.
*/
graph_t* gen_graph_scc( graph_t* graph_v)
{
	int i, pre_idx1, pre_idx2, cur_idx1, cur_idx2;

	graph_t* graph_v1;
	g_node_t* g_node_v1;

	scc_t* scc_v;
	edge_s_t* edge_s_v;

	//Rechieve the SCCs.
	scc_v = calc_scc( graph_v);

	//Rechieve the edge set.
	edge_s_v = gen_edge_set( graph_v, scc_v);

	//Construct the new graph based on the generated information.
	graph_v1 = (graph_t*)malloc( sizeof( graph_t));
	memset( graph_v1, 0, sizeof( graph_t));
	graph_v1->size = scc_v->num;
	graph_v1->head_v = (g_head_t*)malloc( graph_v1->size * sizeof( g_head_t));
	for( i=0; i<graph_v1->size; i++)
	{
		g_node_v1 = (g_node_t*)malloc( sizeof( g_node_t));
		g_node_v1->next = NULL;
		graph_v1->head_v[i].head = g_node_v1;
	}
/*s*/
	sta_v.memory_v += sizeof( graph_t) +
						graph_v1->size * sizeof( g_head_t) +
						graph_v1->size * sizeof( g_node_t);
/*s*/

	pre_idx1 = pre_idx2 = -1;
	for( i=0; i<edge_s_v->num; i++)
	{
		cur_idx1 = edge_s_v->edge_arr[i].idx1;
		cur_idx2 = edge_s_v->edge_arr[i].idx2;

		if( cur_idx1 != pre_idx1 && cur_idx2 != pre_idx2)
		{
			g_node_v1 = (g_node_t*)malloc( sizeof( g_node_t));
			g_node_v1->id = cur_idx2;
			g_node_v1->next = graph_v1->head_v[ cur_idx1].head->next;
			graph_v1->head_v[ cur_idx1].head->next = g_node_v1;
			
			graph_v1->edge_n ++;

			pre_idx1 = cur_idx1;
			pre_idx2 = cur_idx2;
/*s*/
			sta_v.memory_v += sizeof( g_node_t);
/*s*/
		}
	}

/*s*/
	if( sta_v.memory_max < sta_v.memory_v)
		sta_v.memory_max = sta_v.memory_v;
	sta_v.memory_v -= scc_v->size * sizeof( int) +
						sizeof( scc_t) +
						edge_s_v->size * sizeof( edge_t) +
						sizeof( edge_s_t);
/*s*/

	//Release malloced resources.
	free( scc_v->scc_arr);
	free( scc_v);
	free( edge_s_v->edge_arr);
	free( edge_s_v);

	return graph_v1;		
}

/*
*	Count the nodes with in-degree equal to zero.
*	(Multiple methods can do this)
*/
int count_node_zero_in_degree( graph_t* graph_v)
{
	int i, cur_idx, cnt = 0;
	int* in_degree_arr;

	g_node_t* g_node_v;

	in_degree_arr = (int*)malloc( graph_v->size * sizeof( int));
	memset( in_degree_arr, 0, graph_v->size * sizeof( int));
/*s*/
	sta_v.memory_v += graph_v->size * sizeof( int);
	if( sta_v.memory_max < sta_v.memory_v)
		sta_v.memory_max = sta_v.memory_v;
/*s*/

	for( i=0; i<graph_v->size; i++)
	{
		g_node_v = graph_v->head_v[i].head->next;
		while( g_node_v != NULL)
		{
			cur_idx = g_node_v->id;
			in_degree_arr[ cur_idx] ++;

			g_node_v = g_node_v->next;
		}
	}

	for( i=0; i<graph_v->size; i++)
	{
		if( in_degree_arr[i] == 0)
			cnt ++;
	}
/*s*/
	sta_v.memory_v -= graph_v->size * sizeof( int);
/*s*/

	free( in_degree_arr);

	return cnt;
}

/*
*	Solve the seed set minimization problem while influence all the nodes in the graph.
*/
int full_influence_spread_cover( graph_t* graph_v, int sample_n)
{
	int i, portion_n = 1;
	float seed_cnt = 0;

	graph_t* g_deter_v, *g_deter_v1;
	
	for( i=0; i<sample_n; i++)
	{
		//Sampling the graph.
		g_deter_v = graph_sampling_IC( graph_v);

		//Generate the corresponding new graph
		g_deter_v1 = gen_graph_scc( g_deter_v);

		//Count the nodes with in-degree equal to zero in the new graph.
		seed_cnt += (float)count_node_zero_in_degree( g_deter_v1);

		release_graph( g_deter_v);
		release_graph( g_deter_v1);

		if( i >= portion_n * sample_n / MAX_STAGE_NUM)
			printf( "Sampling Stage #%i.\n", portion_n++);
	}
	
	return  (int)ceil(seed_cnt / sample_n);
}

/*----------------------------Testing APIs------------------------------*/

/*
*	test_greedy_alg tests the related greedy algorithms.
* 
* @cfg->alg indicates the implementation option: 
*	1: Greedy1
*	2: Greedy2
*	3: Random
*	4: Degree
*	5: Centrality
* @cfg->model indicates the diffusion model: 1 for IC-Model; 2 for LT-Model.
* 
*/
void test_greedy_alg( )
{
	int min_seed_t;

	config_t* cfg;

	graph_t* g_prob_v;

	FILE* o_fp, *s_fp;
/*s*/
	//Initialize the statistic information.
	memset( &sta_v, 0, sizeof( sta_t));
/*s*/
	if( (o_fp = fopen( MIN_SEED_FILE, "w")) == NULL ||
		(s_fp = fopen( STATISTIC_FILE, "w")) == NULL)
	{
		fprintf( stderr, "Cannot open the min_seed/statistic file.\n");
		exit( 0);
	}

	//Read the configuration information.
	cfg = read_config( );

	//Construct the probabilistic graph.
	g_prob_v = construct_graph( cfg, cfg->model);

	//Perform the corresponding greedy algorithms.
/*s*/
#ifndef WIN32
	struct rusage t_start, t_end;
	float t_s;
	GetCurTime( &t_start);
#endif
/*s*/
	if( cfg->alg == 1)
	{
		printf( "\nGreedy Implementation 1 : Model %i.\n", cfg->model);
		min_seed_t = seed_addition_greedy1_lazy( g_prob_v, cfg->J, cfg->sample_n, cfg->model);
	}
	else if( cfg->alg == 2)
	{
		printf( "\nGreedy Implementation 2 : Model %i.\n", cfg->model);
		min_seed_t = seed_addition_greedy2_lazy( g_prob_v, cfg->J, cfg->sample_n, cfg->model);
	}
	else if( cfg->alg == 3)
	{
		printf( "\nRandom-algorithm : Model %i.\n", cfg->model);
		min_seed_t = random_choose_seed( g_prob_v, cfg->J, cfg->sample_n, cfg->model);
	}
	else if( cfg->alg == 4)
	{
		printf( "\nDegree-heuristic : Model %i.\n", cfg->model);
		min_seed_t = degree_oriented_choose_seed( g_prob_v, cfg->J, cfg->sample_n, cfg->model);
	}
	else if( cfg->alg == 5)
	{
		printf( "\nCentrality-heuristic : Model %i.\n", cfg->model);
		min_seed_t = centrality_oriented_choose_seed( g_prob_v, cfg->J, cfg->sample_n, cfg->model);
	}
	else if( cfg->alg == 6)
	{
		printf( "\nBrute-Force optimization : Model %i.\n", cfg->model);
		min_seed_t = opt_brute_force( g_prob_v, cfg->J, cfg->sample_n, cfg->model);
	}
	else if( cfg->alg == 7)
	{
		printf( "\nFull Influence Coverage.\n");
		min_seed_t = full_influence_spread_cover( g_prob_v, cfg->sample_n);
	}
	else
	{
		fprintf( stderr, "The #Algorithm %i is not supported.\n", cfg->alg);
		exit( 0);
	}

/*s*/
#ifndef WIN32
	GetCurTime( &t_end);
	GetTime( &t_start, &t_end, &(sta_v.exec_time), &t_s);
#endif
/*s*/

	//Output the result & statistic information.
	fprintf( o_fp, "%i\n", min_seed_t);
	fprintf( s_fp, "%f\n", sta_v.exec_time);
	fprintf( s_fp, "%f\n", sta_v.memory_max);

	//Release the related resources.
	free( cfg);
	release_graph( g_prob_v);
	fclose( o_fp);
	fclose( s_fp);

	printf( "Memory: %f\n", sta_v.memory_v); 
}
